//////////////////////////////////////////////////////////////////////////////
// Filename:          Y:\Usagers\jjohnson\Projects\MGTOscillator2/drivers/oscillator_v1_00_a/src/oscillator.h
// Version:           1.00.a
// Description:       oscillator Driver Header File
// Date:              Wed Mar 26 14:53:14 2008 (by Create and Import Peripheral Wizard)
//////////////////////////////////////////////////////////////////////////////

#ifndef OSCILLATOR_H
#define OSCILLATOR_H

/***************************** Include Files *******************************/

#include "xbasic_types.h"
#include "xstatus.h"
#include "xio.h"

/************************** Constant Definitions ***************************/


/**
 * User Logic Slave Space Offsets
 * -- SLAVE_REG0 : user logic slave module register 0
 * -- SLAVE_REG1 : user logic slave module register 1
 * -- SLAVE_REG2 : user logic slave module register 2
 * -- SLAVE_REG3 : user logic slave module register 3
 */
#define OSCILLATOR_USER_SLAVE_SPACE_OFFSET (0x00000000)
#define OSCILLATOR_SLAVE_REG0_OFFSET (OSCILLATOR_USER_SLAVE_SPACE_OFFSET + 0x00000000)
#define OSCILLATOR_SLAVE_REG1_OFFSET (OSCILLATOR_USER_SLAVE_SPACE_OFFSET + 0x00000004)
#define OSCILLATOR_SLAVE_REG2_OFFSET (OSCILLATOR_USER_SLAVE_SPACE_OFFSET + 0x00000008)
#define OSCILLATOR_SLAVE_REG3_OFFSET (OSCILLATOR_USER_SLAVE_SPACE_OFFSET + 0x0000000C)

/**************************** Type Definitions *****************************/


/***************** Macros (Inline Functions) Definitions *******************/

/**
 *
 * Write a value to a OSCILLATOR register. A 32 bit write is performed.
 * If the component is implemented in a smaller width, only the least
 * significant data is written.
 *
 * @param   BaseAddress is the base address of the OSCILLATOR device.
 * @param   RegOffset is the register offset from the base to write to.
 * @param   Data is the data written to the register.
 *
 * @return  None.
 *
 * @note
 * C-style signature:
 * 	void OSCILLATOR_mWriteReg(Xuint32 BaseAddress, unsigned RegOffset, Xuint32 Data)
 *
 */
#define OSCILLATOR_mWriteReg(BaseAddress, RegOffset, Data) \
 	XIo_Out32((BaseAddress) + (RegOffset), (Xuint32)(Data))

/**
 *
 * Read a value from a OSCILLATOR register. A 32 bit read is performed.
 * If the component is implemented in a smaller width, only the least
 * significant data is read from the register. The most significant data
 * will be read as 0.
 *
 * @param   BaseAddress is the base address of the OSCILLATOR device.
 * @param   RegOffset is the register offset from the base to write to.
 *
 * @return  Data is the data from the register.
 *
 * @note
 * C-style signature:
 * 	Xuint32 OSCILLATOR_mReadReg(Xuint32 BaseAddress, unsigned RegOffset)
 *
 */
#define OSCILLATOR_mReadReg(BaseAddress, RegOffset) \
 	XIo_In32((BaseAddress) + (RegOffset))


/**
 *
 * Write/Read value to/from OSCILLATOR user logic slave registers.
 *
 * @param   BaseAddress is the base address of the OSCILLATOR device.
 * @param   Value is the data written to the register.
 *
 * @return  Data is the data from the user logic slave register.
 *
 * @note
 * C-style signature:
 * 	Xuint32 OSCILLATOR_mReadSlaveRegn(Xuint32 BaseAddress)
 *
 */
#define OSCILLATOR_mWriteSlaveReg0(BaseAddress, Value) \
 	XIo_Out32((BaseAddress) + (OSCILLATOR_SLAVE_REG0_OFFSET), (Xuint32)(Value))
#define OSCILLATOR_mWriteSlaveReg1(BaseAddress, Value) \
 	XIo_Out32((BaseAddress) + (OSCILLATOR_SLAVE_REG1_OFFSET), (Xuint32)(Value))
#define OSCILLATOR_mWriteSlaveReg2(BaseAddress, Value) \
 	XIo_Out32((BaseAddress) + (OSCILLATOR_SLAVE_REG2_OFFSET), (Xuint32)(Value))
#define OSCILLATOR_mWriteSlaveReg3(BaseAddress, Value) \
 	XIo_Out32((BaseAddress) + (OSCILLATOR_SLAVE_REG3_OFFSET), (Xuint32)(Value))

#define OSCILLATOR_mReadSlaveReg0(BaseAddress) \
 	XIo_In32((BaseAddress) + (OSCILLATOR_SLAVE_REG0_OFFSET))
#define OSCILLATOR_mReadSlaveReg1(BaseAddress) \
 	XIo_In32((BaseAddress) + (OSCILLATOR_SLAVE_REG1_OFFSET))
#define OSCILLATOR_mReadSlaveReg2(BaseAddress) \
 	XIo_In32((BaseAddress) + (OSCILLATOR_SLAVE_REG2_OFFSET))
#define OSCILLATOR_mReadSlaveReg3(BaseAddress) \
 	XIo_In32((BaseAddress) + (OSCILLATOR_SLAVE_REG3_OFFSET))

/************************** Function Prototypes ****************************/


/**
 *
 * Run a self-test on the driver/device. Note this may be a destructive test if
 * resets of the device are performed.
 *
 * If the hardware system is not built correctly, this function may never
 * return to the caller.
 *
 * @param   baseaddr_p is the base address of the OSCILLATOR instance to be worked on.
 *
 * @return
 *
 *    - XST_SUCCESS   if all self-test code passed
 *    - XST_FAILURE   if any self-test code failed
 *
 * @note    Caching must be turned off for this function to work.
 * @note    Self test may fail if data memory and device are not on the same bus.
 *
 */
XStatus OSCILLATOR_SelfTest(void * baseaddr_p);

#endif // OSCILLATOR_H
