------------------------------------------------------------------------------
-- user_logic.vhd - entity/architecture pair
------------------------------------------------------------------------------
--
-- ***************************************************************************
-- ** Copyright (c) 1995-2006 Xilinx, Inc.  All rights reserved.            **
-- **                                                                       **
-- ** Xilinx, Inc.                                                          **
-- ** XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"         **
-- ** AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND       **
-- ** SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,        **
-- ** OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,        **
-- ** APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION           **
-- ** THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,     **
-- ** AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE      **
-- ** FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY              **
-- ** WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE               **
-- ** IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR        **
-- ** REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF       **
-- ** INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS       **
-- ** FOR A PARTICULAR PURPOSE.                                             **
-- **                                                                       **
-- ***************************************************************************
--
------------------------------------------------------------------------------
-- Filename:          user_logic.vhd
-- Version:           1.00.a
-- Description:       User logic.
-- Date:              Wed Mar 26 14:52:55 2008 (by Create and Import Peripheral Wizard)
-- VHDL Standard:     VHDL'93
------------------------------------------------------------------------------
-- Naming Conventions:
--   active low signals:                    "*_n"
--   clock signals:                         "clk", "clk_div#", "clk_#x"
--   reset signals:                         "rst", "rst_n"
--   generics:                              "C_*"
--   user defined types:                    "*_TYPE"
--   state machine next state:              "*_ns"
--   state machine current state:           "*_cs"
--   combinatorial signals:                 "*_com"
--   pipelined or register delay signals:   "*_d#"
--   counter signals:                       "*cnt*"
--   clock enable signals:                  "*_ce"
--   internal version of output port:       "*_i"
--   device pins:                           "*_pin"
--   ports:                                 "- Names begin with Uppercase"
--   processes:                             "*_PROCESS"
--   component instantiations:              "<ENTITY_>I_<#|FUNC>"
------------------------------------------------------------------------------

-- DO NOT EDIT BELOW THIS LINE --------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library proc_common_v2_00_a;
use proc_common_v2_00_a.proc_common_pkg.all;
-- DO NOT EDIT ABOVE THIS LINE --------------------

--USER libraries added here

------------------------------------------------------------------------------
-- Entity section
------------------------------------------------------------------------------
-- Definition of Generics:
--   C_DWIDTH                     -- User logic data bus width
--   C_NUM_CE                     -- User logic chip enable bus width
--
-- Definition of Ports:
--   Bus2IP_Clk                   -- Bus to IP clock
--   Bus2IP_Reset                 -- Bus to IP reset
--   Bus2IP_Data                  -- Bus to IP data bus for user logic
--   Bus2IP_BE                    -- Bus to IP byte enables for user logic
--   Bus2IP_RdCE                  -- Bus to IP read chip enable for user logic
--   Bus2IP_WrCE                  -- Bus to IP write chip enable for user logic
--   IP2Bus_Data                  -- IP to Bus data bus for user logic
--   IP2Bus_Ack                   -- IP to Bus acknowledgement
--   IP2Bus_Retry                 -- IP to Bus retry response
--   IP2Bus_Error                 -- IP to Bus error response
--   IP2Bus_ToutSup               -- IP to Bus timeout suppress
------------------------------------------------------------------------------

entity user_logic is
  generic
  (
    -- ADD USER GENERICS BELOW THIS LINE ---------------
    --USER generics added here
    -- ADD USER GENERICS ABOVE THIS LINE ---------------

    -- DO NOT EDIT BELOW THIS LINE ---------------------
    -- Bus protocol parameters, do not add to or delete
    C_DWIDTH                       : integer              := 32;
    C_NUM_CE                       : integer              := 4
    -- DO NOT EDIT ABOVE THIS LINE ---------------------
  );
  port
  (
    -- ADD USER PORTS BELOW THIS LINE ------------------
  TOP_BREF_CLK   : in std_logic;
  USER_CLK       : in std_logic;
  RXP            : in std_logic;
  RXN            : in std_logic;
  TXP            : out std_logic;
  TXN            : out std_logic;
    -- ADD USER PORTS ABOVE THIS LINE ------------------

    -- DO NOT EDIT BELOW THIS LINE ---------------------
    -- Bus protocol ports, do not add to or delete
    Bus2IP_Clk                     : in  std_logic;
    Bus2IP_Reset                   : in  std_logic;
    Bus2IP_Data                    : in  std_logic_vector(0 to C_DWIDTH-1);
    Bus2IP_BE                      : in  std_logic_vector(0 to C_DWIDTH/8-1);
    Bus2IP_RdCE                    : in  std_logic_vector(0 to C_NUM_CE-1);
    Bus2IP_WrCE                    : in  std_logic_vector(0 to C_NUM_CE-1);
    IP2Bus_Data                    : out std_logic_vector(0 to C_DWIDTH-1);
    IP2Bus_Ack                     : out std_logic;
    IP2Bus_Retry                   : out std_logic;
    IP2Bus_Error                   : out std_logic;
    IP2Bus_ToutSup                 : out std_logic
    -- DO NOT EDIT ABOVE THIS LINE ---------------------
  );
end entity user_logic;

------------------------------------------------------------------------------
-- Architecture section
------------------------------------------------------------------------------

architecture IMP of user_logic is

  --USER signal declarations added here, as needed for user logic
signal mgt_txdata : std_logic_vector(19 downto 0);
signal count      : std_logic_vector(3 downto 0);
 
component GT_CUSTOM
   generic(
     REF_CLK_V_SEL : integer);
   port(
     CHBONDDONE     : out std_logic;
     CHBONDO        : out std_logic_vector(3 downto 0);
     CONFIGOUT      : out std_logic;
     RXBUFSTATUS    : out std_logic_vector(1 downto 0);
     RXCHARISCOMMA  : out std_logic_vector(3 downto 0);
     RXCHARISK      : out std_logic_vector(3 downto 0);
     RXCHECKINGCRC  : out std_logic;
     RXCLKCORCNT    : out std_logic_vector(2 downto 0);
     RXCOMMADET     : out std_logic;
     RXCRCERR       : out std_logic;
     RXDATA         : out std_logic_vector(31 downto 0);
     RXDISPERR      : out std_logic_vector(3 downto 0);
     RXLOSSOFSYNC   : out std_logic_vector(1 downto 0);
     RXNOTINTABLE   : out std_logic_vector(3 downto 0);
     RXREALIGN      : out std_logic;
     RXRECCLK       : out std_logic;
     RXRUNDISP      : out std_logic_vector(3 downto 0);
     TXBUFERR       : out std_logic;
     TXKERR         : out std_logic_vector(3 downto 0);
     TXN            : out std_logic;
     TXP            : out std_logic;
     TXRUNDISP      : out std_logic_vector(3 downto 0);
     BREFCLK        : in std_logic;
     BREFCLK2       : in std_logic;
     CHBONDI        : in std_logic_vector(3 downto 0);
     CONFIGENABLE   : in std_logic;
     CONFIGIN       : in std_logic;
     ENCHANSYNC     : in std_logic;
     ENMCOMMAALIGN  : in std_logic;
     ENPCOMMAALIGN  : in std_logic;
     LOOPBACK       : in std_logic_vector(1 downto 0);
     POWERDOWN      : in std_logic;
     REFCLK         : in std_logic;
     REFCLK2        : in std_logic;
     REFCLKSEL      : in std_logic;
     RXN            : in std_logic;
     RXP            : in std_logic;
     RXPOLARITY     : in std_logic;
     RXRESET        : in std_logic;
     RXUSRCLK       : in std_logic;
     RXUSRCLK2      : in std_logic;
     TXBYPASS8B10B  : in std_logic_vector(3 downto 0);
     TXCHARDISPMODE : in std_logic_vector(3 downto 0);
     TXCHARDISPVAL  : in std_logic_vector(3 downto 0);
     TXCHARISK      : in std_logic_vector(3 downto 0);
     TXDATA         : in std_logic_vector(31 downto 0);
     TXFORCECRCERR  : in std_logic;
     TXINHIBIT      : in std_logic;
     TXPOLARITY     : in std_logic;
     TXRESET        : in std_logic;
     TXUSRCLK       : in std_logic;
     TXUSRCLK2      : in std_logic);
end component;

  ------------------------------------------
  -- Signals for user logic slave model s/w accessible register example
  ------------------------------------------
  signal slv_reg0                       : std_logic_vector(0 to C_DWIDTH-1);
  signal slv_reg1                       : std_logic_vector(0 to C_DWIDTH-1);
  signal slv_reg2                       : std_logic_vector(0 to C_DWIDTH-1);
  signal slv_reg3                       : std_logic_vector(0 to C_DWIDTH-1);
  signal slv_reg_write_select           : std_logic_vector(0 to 3);
  signal slv_reg_read_select            : std_logic_vector(0 to 3);
  signal slv_ip2bus_data                : std_logic_vector(0 to C_DWIDTH-1);
  signal slv_read_ack                   : std_logic;
  signal slv_write_ack                  : std_logic;

begin

  --USER logic implementation added here
rock0 : GT_CUSTOM
  generic map(
    REF_CLK_V_SEL              => 1)
  port map(
    CHBONDDONE                 => open,
    CHBONDI                    => "0000",
    CHBONDO                    => open,
    CONFIGENABLE               => '0',
    CONFIGIN                   => '0',
    CONFIGOUT                  => open,
    ENCHANSYNC                 => '0',
    ENMCOMMAALIGN              => '0',
    ENPCOMMAALIGN              => '0',
    LOOPBACK                   => "00",
    POWERDOWN                  => '0',
    BREFCLK                    => TOP_BREF_CLK,
    BREFCLK2                   => '0',
    REFCLK                     => '0',
    REFCLK2                    => '0',
    REFCLKSEL                  => '0',
    RXBUFSTATUS                => open,
    RXCHARISCOMMA              => open,
    RXCHARISK                  => open,
    RXCHECKINGCRC              => open,
    RXCLKCORCNT                => open,
    RXCOMMADET                 => open,
    RXCRCERR                   => open,
    RXDATA                     => open,
    RXDISPERR                  => open,
    RXLOSSOFSYNC               => open,
    RXN                        => RXN,
    RXNOTINTABLE               => open,
    RXP                        => RXP,
    RXPOLARITY                 => '0',
    RXREALIGN                  => open,
    RXRECCLK                   => open,
    RXRESET                    => '0',
    RXRUNDISP                  => open,
    RXUSRCLK                   => '0',
    RXUSRCLK2                  => '0',
    TXBUFERR                   => open,
    TXBYPASS8B10B              => "1111",
    TXCHARDISPMODE(3 downto 2) => "00",
    TXCHARDISPMODE(1)          => mgt_txdata(19),
    TXCHARDISPMODE(0)          => mgt_txdata(9),
    TXCHARDISPVAL(3 downto 2)  => "00",
    TXCHARDISPVAL(1)           => mgt_txdata(18),
    TXCHARDISPVAL(0)           => mgt_txdata(8),
    TXCHARISK(3 downto 2)      => "00",
    TXCHARISK(1 downto 0)      => "00",
    TXDATA(31 downto 16)       => "0000000000000000",
    TXDATA(15 downto 8)        => mgt_txdata(17 downto 10),
    TXDATA(7 downto 0)         => mgt_txdata(7 downto 0),
    TXFORCECRCERR              => '0',
    TXINHIBIT                  => '0',
    TXKERR                     => open,
    TXN                        => TXN,
    TXP                        => TXP,
    TXPOLARITY                 => '0',
    TXRESET                    => Bus2IP_Reset,
    TXRUNDISP                  => open,
    TXUSRCLK                   => USER_CLK,
    TXUSRCLK2                  => USER_CLK);

-- This process manages a counter and feeds the MGT
-- with data from the slave registers  
process (USER_CLK, Bus2IP_Reset)
begin
  if Bus2IP_Reset = '1' then
    mgt_txdata <= (others => '0');
    count <= (others => '0');
  elsif USER_CLK'event and USER_CLK = '1' then
    if count = "0011" then
      count <= (others => '0');
    else
      count <= count + 1;
    end if;
    case count is 
      when "0000" => mgt_txdata <= slv_reg0(12 to 31);
      when "0001" => mgt_txdata <= slv_reg1(12 to 31);
      when "0010" => mgt_txdata <= slv_reg2(12 to 31);
      when others => mgt_txdata <= slv_reg3(12 to 31);
    end case;
  end if;
end process;

  ------------------------------------------
  -- Example code to read/write user logic slave model s/w accessible registers
  -- 
  -- Note:
  -- The example code presented here is to show you one way of reading/writing
  -- software accessible registers implemented in the user logic slave model.
  -- Each bit of the Bus2IP_WrCE/Bus2IP_RdCE signals is configured to correspond
  -- to one software accessible register by the top level template. For example,
  -- if you have four 32 bit software accessible registers in the user logic, you
  -- are basically operating on the following memory mapped registers:
  -- 
  --    Bus2IP_WrCE or   Memory Mapped
  --       Bus2IP_RdCE   Register
  --            "1000"   C_BASEADDR + 0x0
  --            "0100"   C_BASEADDR + 0x4
  --            "0010"   C_BASEADDR + 0x8
  --            "0001"   C_BASEADDR + 0xC
  -- 
  ------------------------------------------
  slv_reg_write_select <= Bus2IP_WrCE(0 to 3);
  slv_reg_read_select  <= Bus2IP_RdCE(0 to 3);
  slv_write_ack        <= Bus2IP_WrCE(0) or Bus2IP_WrCE(1) or Bus2IP_WrCE(2) or Bus2IP_WrCE(3);
  slv_read_ack         <= Bus2IP_RdCE(0) or Bus2IP_RdCE(1) or Bus2IP_RdCE(2) or Bus2IP_RdCE(3);

  -- implement slave model register(s)
  SLAVE_REG_WRITE_PROC : process( Bus2IP_Clk ) is
  begin

    if Bus2IP_Clk'event and Bus2IP_Clk = '1' then
      if Bus2IP_Reset = '1' then
        slv_reg0 <= (others => '0');
        slv_reg1 <= (others => '0');
        slv_reg2 <= (others => '0');
        slv_reg3 <= (others => '0');
      else
        case slv_reg_write_select is
          when "1000" =>
            for byte_index in 0 to (C_DWIDTH/8)-1 loop
              if ( Bus2IP_BE(byte_index) = '1' ) then
                slv_reg0(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
              end if;
            end loop;
          when "0100" =>
            for byte_index in 0 to (C_DWIDTH/8)-1 loop
              if ( Bus2IP_BE(byte_index) = '1' ) then
                slv_reg1(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
              end if;
            end loop;
          when "0010" =>
            for byte_index in 0 to (C_DWIDTH/8)-1 loop
              if ( Bus2IP_BE(byte_index) = '1' ) then
                slv_reg2(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
              end if;
            end loop;
          when "0001" =>
            for byte_index in 0 to (C_DWIDTH/8)-1 loop
              if ( Bus2IP_BE(byte_index) = '1' ) then
                slv_reg3(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
              end if;
            end loop;
          when others => null;
        end case;
      end if;
    end if;

  end process SLAVE_REG_WRITE_PROC;

  -- implement slave model register read mux
  SLAVE_REG_READ_PROC : process( slv_reg_read_select, slv_reg0, slv_reg1, slv_reg2, slv_reg3 ) is
  begin

    case slv_reg_read_select is
      when "1000" => slv_ip2bus_data <= slv_reg0;
      when "0100" => slv_ip2bus_data <= slv_reg1;
      when "0010" => slv_ip2bus_data <= slv_reg2;
      when "0001" => slv_ip2bus_data <= slv_reg3;
      when others => slv_ip2bus_data <= (others => '0');
    end case;

  end process SLAVE_REG_READ_PROC;

  ------------------------------------------
  -- Example code to drive IP to Bus signals
  ------------------------------------------
  IP2Bus_Data        <= slv_ip2bus_data;

  IP2Bus_Ack         <= slv_write_ack or slv_read_ack;
  IP2Bus_Error       <= '0';
  IP2Bus_Retry       <= '0';
  IP2Bus_ToutSup     <= '0';

end IMP;
